#!/bin/bash
trap 'trap_handler' SIGINT 
set -m	#turn off job control -- dont send sgint to children! when strg+c was received!

TEMP=`getopt -o s:e:c:h --long start:,end:,channels:,callingPartyNumber:,good:,bad:,help -n "$0" -- "$@"`
[ $? != 0 ] && {
    echo "Terminating..." >&2
        exit 1
}

##### edit me ####
start=0
end=20000
channels=30
sippStartPort=6000
rtpStartPort=2000
callingPartyNumber="\#9"
good=logs/pbx.list
bad=logs/400.list
killer=52		#kill any sipp instance after N seconds - (prevent dead sipps)		# set it high enough! ~30-40s
##### ############



iface=$(LC_ALL=C route -n | grep 0.0.0.0 | grep UG | awk -F " " '{print $8}')
[ -z ${iface} ] && iface=eth0
IP=$(LC_ALL=C ifconfig $iface | grep inet\ addr | awk -F ":" '{print $2}' | sed s/\ .*//g)

mkdir -p logs
number=${start}
declare -a pids
before=$(date +%s)
count=0
sippport=${sippStartPort}
rtpport=${rtpStartPort}


function trap_handler(){
	echo
	echo
	echo "trap handler called !! $0 $1 $2 $3            ##  STRG+C was pressed"
    echo Waiting for remaining sipps to finish ${pids[*]}
    number=$((${end}+10))
	while [ ${#pids[*]} -ne 0 ]; do
		check_pids
		echo waiting for: ${pids[*]}
		sleep 1
	done
	formatLogs
	after=$(date +%s)
    echo
    echo "completed: ${count} tests on $channels channels"
    echo "elapsed time:" $((after - $before)) "seconds"
exit 0
}

usage(){
cat << XXX
        $0 usage:
			press STRG+C for a long time to abort
			disable rsyslog: service rsyslog stop
XXX
exit 1
}


function formatLogs(){
	blwl c ${good} ${good}-blwl.list
	blwl c ${bad} ${bad}-blwl.list
}

function getFreeSippPort(){
	sleep 0.2
	tcpPort=$sippStartPort
	while netstat -atwn | grep -q "^tcp.*:${tcpPort}"; do
			tcpPort=$(( ${tcpPort} + 1 ))
	done
	sippport=${tcpPort}
}
function getFreeRtpPort(){
	sleep 0.2
	udpPort=${rtpStartPort}
	while netstat -auwn | grep -q "^udp.*:${udpPort}"; do
			udpPort=$(( ${udpPort} + 4 ))
	done
	rtpport=${udpPort}
}

function check_pids(){	
	unset tmpPids		#declare empty array
		declare -a tmpPids
		for i in ${!pids[@]}; do
            isAlive=$(kill -0 ${pids[$i]} 2>/dev/null;echo $?)		# check if pid is alive
			if [ "$isAlive" == "1" ];then
				unset pids[i]			# remove dead pid
			fi
		done
		c=0
		for i in ${!pids[@]}; do		# put all pids that are not "" -empty on the new array 
			if [ ! -z ${pids[$i]} ];then
				tmpPids[$c]=${pids[$i]}
			fi
			c=$(($c+1))
		done
		unset pids						# delete the old array
		pids=("${tmpPids[@]}")			# copy new array over the old one
#		echo ${pids[*]}
		sleep 0.3
}
function pidExists(){
	for i in ${pids[@]}; do
		if [ $i == $1 ]; then
			return 0
		fi
	done
	return 1		#return 1 if the element does not exists
}


function waitForPorts(){
	pid=$1
	sippport=$2
	rtpport=$3
	while [ $(netstat -atwn | grep  -q "^tcp.*:${sippport}"; echo $? ) -ne 0 ] ; do		
		check_pids			#remove dead pids
		if  pidExists $pid ; then 		# is this pid still in the pids list?
			sleep 0.3	## wait for sipp to start and bind the port
		else
			echo "PID DIED VERY QUICKLY!!!! port in use? number: $number ; sipport: $sippport rtppport: $rtpport"	>> errors.log
			break	#abort
		fi
    done
	while [ $(netstat -auwn | grep  -q "^udp.*:${rtpport}"; echo $? ) -ne 0 ] ; do		
		check_pids	
		if  pidExists $pid ; then 		# is this pid still in the pids list?
			sleep 0.3	## wait for sipp to start and bind the port
		else	
			echo "PID DIED VERY QUICKLY!!!! port in use? number: $number ; sipport: $sippport rtppport: $rtpport"	>> errors.log
			break	#abort
		fi

		sleep 0.3	## wait for sipp to start and bind the port
    done
}
#start=0
#end=20000
#channels=30
#sippStartPort=6000
#rtpStartPort=2000
#callingPartyNumber="\#9"
#good=logs/pbx.list
#bad=logs/400.list


#[ $# -lt 3 ] && usage
while false ; do				##falsified
    case "$1" in
        -s|--start)
			start+=("$2")
            shift
            ;;
        -e|--end)
            end+=("$2")
            shift
            ;;
        -c|--channnels)
            channels+=("$2")
			shift
            ;;
        --callingPartyNumber)
            callingPartyNumber+=("$2")
            shift
            ;;
	    --good)
            good+=("$2")
			shift
            ;;
        --bad)
            bad+=("$2")
            shift
            ;;
        -h|--help)
            HELP=1
            ;;
        --)
        shift
        break;;
#        *) echo "1st: $1 2nd: $2 3rd: $3 4th: $4" 5th: $5 6th: $6; exit 1 ;;
    esac
        shift
done
[ ! -z ${HELP} ] && usage




while [ true ];do
	if [ ${#pids[*]} -lt ${channels} ]; then
        if [ ${number} -le ${end} ]; then
			count=$(($count+1))
			getFreeSippPort
		 	getFreeRtpPort
	
#		    echo "======calling from port:$sippport with rtpport:$rtpport to number: $number" 
			echo -n "..$number::$sippport::$rtpport.."
			cmd() {
				# -trace_dtmf -trace_logs -rtp_receive -rtp_echo -trace_shortmsg -trace_msg -trace_err   
				## kill sipp after running 15s interval for checks: 2s, wait 1s before killing
				./timeout3 -t ${killer} -i 2 -d 1 \
					 sipp 127.0.0.1:5060 -sf bruteCall.xml -mp ${rtpport}  -p ${sippport} -set callingPartyNumber ${callingPartyNumber} -silent \
					 -t tn -i ${IP} -mi ${IP} -pause_msg_ign -m 1 -s ${number} -trace_msg -trace_shortmsg 2>&1 > logs/log_${number}.log 2>&1
                    ret=$?
                    [ $ret -eq 0 ] && echo ${number} >> ${good}
                    [ $ret -ne 0 ] && echo ${number} >> ${bad}
		#		echo "REEEET : $ret"	
				}
			cmd &
			pid=$!

			pids[${#pids[*]}]=$pid
	 		waitForPorts $pid ${sippport} ${rtpport}


            number=$((${number}+1))
		else	# number gt end
			echo "done"; 
			echo "### wait for remaining ###"
   			while [ ${#pids[*]} -ne 0 ]; do
				echo waiting for: ${pids[*]}
				check_pids
				sleep 1
			done
			rm -rf *.log
			formatLogs
			after=$(date +%s)
            echo
            echo "completed: ${count} tests on $channels channels"
            echo "elapsed time:" $((after - $before)) "seconds"

			#exit 0
			break
		fi


	else	#pids > channels	
		rm -f *.log
		check_pids
		sleep 0.1
	fi
done

